package com.flyer.foster.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.bean.copier.CopyOptions;
import com.flyer.foster.entity.Menu;
import com.flyer.foster.entity.Role;
import com.flyer.foster.entity.RoleMenu;
import com.flyer.foster.entity.User;
import com.flyer.foster.enums.DeleteStatusEnum;
import com.flyer.foster.enums.IsAdminEnum;
import com.flyer.foster.enums.StatusEnum;
import com.flyer.foster.mapper.IMenuMapper;
import com.flyer.foster.pojo.CustomMeta;
import com.flyer.foster.pojo.RoleAssignTreeMenu;
import com.flyer.foster.pojo.TreeAllMenu;
import com.flyer.foster.pojo.TreeMenu;
import com.flyer.foster.service.IMenuService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.flyer.foster.service.IRoleMenuService;
import com.flyer.foster.service.IRoleService;
import com.flyer.foster.service.IUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * 菜单表 服务实现类
 * </p>
 *
 * @author flyer
 * @since 2024-05-06
 */
@Service
public class MenuServiceImpl extends ServiceImpl<IMenuMapper, Menu> implements IMenuService {

    @Autowired
    private IUserService iUserService;

    @Autowired
    private IRoleService iRoleService;

    @Autowired
    private IRoleMenuService iRoleMenuService;

    @Override
    public List<String> selectPermissionsByUserId(Integer userId) {
        return baseMapper.selectPermissionsByUserId(userId);
    }

    @Override
    public List<TreeMenu> selectMenusByUserName(String userName) {
        User user = iUserService.lambdaQuery().eq(User::getUsername, userName).one();
        // 查询所有菜单，包括了type=2的数据
        List<TreeMenu> menus = baseMapper.selectListByUserIdConvert2DTO(user.getId());
        // 过滤根节点
        List<TreeMenu> roots = menus.stream().filter(x -> x.getParentId().equals(0)).collect(Collectors.toList());
        for (TreeMenu root : roots) {
            this.buildMenus(root, menus);
        }
        return roots;
    }

    @Override
    public List<TreeAllMenu> selectMenusByRoleId(Integer roleId) {
        List<TreeAllMenu> menus = baseMapper.selectListByRoleIdConvert2DTO(roleId);
        // 过滤根节点
        List<TreeAllMenu> roots = menus.stream().filter(x -> x.getParentId().equals(0)).collect(Collectors.toList());
        for (TreeAllMenu root : roots) {
            this.buildAllMenus(root, menus);
        }
        return roots;
    }

    @Override
    public List<TreeAllMenu> getAllTypeMenuList() {
        Role adminRole = iRoleService.lambdaQuery().eq(Role::getIsAdmin, 1).one();
        List<RoleMenu> roleMenuList = iRoleMenuService.lambdaQuery().eq(RoleMenu::getRoleId, adminRole.getId()).list();
        List<TreeAllMenu> menus = baseMapper.selectAllTypeMenus(roleMenuList.stream().map(RoleMenu::getMenuId).collect(Collectors.toList()));
        // 过滤根节点
        List<TreeAllMenu> roots = menus.stream().filter(x -> x.getParentId().equals(0)).collect(Collectors.toList());
        for (TreeAllMenu root : roots) {
            this.assignMenu(root, menus);
        }
        return roots;
    }

    /**
     * 递归构建菜单数据结构
     *
     * @return list
     */
    private void buildMenus(TreeMenu root, List<TreeMenu> data) {
        CustomMeta meta = new CustomMeta();
        for (TreeMenu menu : data) {
            if (menu.getParentId().equals(root.getId())) {
                if (menu.getType() == 1) {
                    root.getChildren().add(menu);
                } else if (menu.getType() == 2) {
                    meta.getPermission().add(menu.getPermission());
                }
            }
        }
        root.setMeta(meta);
        for (TreeMenu child : root.getChildren()) {
            this.buildMenus(child, data);
        }
    }

    /**
     * 构建角色分配菜单弹出框的数据结构
     *
     * @param root
     * @param data
     */
    private void buildAllMenus(TreeAllMenu root, List<TreeAllMenu> data) {
        for (TreeAllMenu menu : data) {
            if (menu.getParentId().equals(root.getId())) {
                root.getChildren().add(menu);
            }
        }
        for (TreeAllMenu child : root.getChildren()) {
            this.buildAllMenus(child, data);
        }
    }

    /**
     * 给角色分配菜单的数据，过滤掉权限管理
     *
     * @return list
     */
    private void assignMenu(TreeAllMenu root, List<TreeAllMenu> data) {
        for (TreeAllMenu menu : data) {
            if (!menu.getTitle().equals("权限管理") && menu.getParentId().equals(root.getId())) {
                root.getChildren().add(menu);
            }
        }
        for (TreeAllMenu child : root.getChildren()) {
            this.buildAllMenus(child, data);
        }
    }

    @Override
    public List<RoleAssignTreeMenu> getRoleAssignTreeMenuList() {
        List<Menu> menuList = this.lambdaQuery()
                .eq(Menu::getIsDeleted, DeleteStatusEnum.UNDELETED.getId())
                .eq(Menu::getStatus, StatusEnum.ENABLE.getId())
                .eq(Menu::getIsAdmin, IsAdminEnum.NO.getId())
                .orderByAsc(Menu::getMenuSort)
                .list();
        List<RoleAssignTreeMenu> list = new ArrayList<>();
        RoleAssignTreeMenu roleAssignTreeMenu;
        for (Menu menu : menuList) {
            roleAssignTreeMenu = new RoleAssignTreeMenu();
            Map<String, String> mapping = new HashMap<>();
            mapping.put("menuName", "title");
            BeanUtil.copyProperties(menu, roleAssignTreeMenu, CopyOptions.create().setFieldMapping(mapping));
            list.add(roleAssignTreeMenu);
        }
        List<RoleAssignTreeMenu> roots = list.stream().filter(x -> x.getParentId().equals(0)).collect(Collectors.toList());
        for (RoleAssignTreeMenu root : roots) {
            this.buildRoleAssignTreeMenuList(root, list);
        }
        return roots;
    }

    /**
     * 角色分配菜单，构建所有菜单的树形结构
     *
     * @param root 根节点
     * @param list 所有菜单集合
     */
    private void buildRoleAssignTreeMenuList(RoleAssignTreeMenu root, List<RoleAssignTreeMenu> list) {
        for (RoleAssignTreeMenu menu : list) {
            if (menu.getParentId().equals(root.getId())) {
                root.getChildren().add(menu);
            }
        }
        for (RoleAssignTreeMenu child : root.getChildren()) {
            this.buildRoleAssignTreeMenuList(child, list);
        }
    }
}
